##############################################################################
# PROGRAM	MRAT_scoring.py
# PURPOSE
# Assign scores to HUC polygons to determine probability and certainty that
# a Tungsten skarn may be present.
#
##############################################################################

# Import system modules
print "Starting MRAT script..."
import sys, traceback, os, arcpy
from arcpy import env

# Change backslash to forward slash in pathname
def fixpath(path):
  path2 = path.replace("\\", "/")
  return path2

try:
  print "Setting script variables..."
  dirpath = "Q:/AKCoop/GISdata_current"
  workpath = "Q:/AKCoop/Projects/MRAT_working"
  
  env.overwriteOutput = 1
  env.XYTolerance = 0.001
  
# Input data sources
  ardf_points = workpath + "/Scored_tracts.gdb/Inputs/ARDF_W_skarn_ranked"
  sed_points = dirpath + "/Geochemistry/Geochemistry.gdb/seds/sed_BV"
  sed_points1 = dirpath + "/Geochemistry/FY19_21_reanalyses.gdb/sed_BV_FY19_reanalyzed"
  rck_points = dirpath + "/Geochemistry/Geochemistry.gdb/rock/rock_BV"
  hmc_points = dirpath + "/Geochemistry/Geochemistry.gdb/HMC/HMC_BV"
  mineral_points = dirpath + "/Geochemistry/Geochemistry.gdb/HMC/HMC_mnrlgy"
  
  temp_path = "in_memory/"
  output_path = workpath + "/Scored_tracts.gdb/Output_scores/"

  tract_polys_orig = workpath + "/Scored_tracts.gdb/Inputs/finalTract_polished_output1_HUCS"
  tract_polys = "in_memory/W_skarn_tractfinal_scores"

# Set projection
  env.outputCoordinateSystem = tract_polys_orig

# Lists of fields to add to the HUC polygons 
  score_fields = [["ARDF_SCORE","#"],
                  ["SED_SCORE","SEDIMENT_SCORE"],
                  ["SED_19_SCORE","SEDIMENT_19_SCORE"],
                  ["RCK_SCORE","ROCK_SCORE"],
                  ["HMC_SCORE","HEAVY_MIN_CONC_SCORE"],
                  ["MIN_SCORE","MINERAL_SCORE"],
                  ["TOTAL_SCORE","#"],
                  ["SED_W_SCORE","#"],
                  ["SED_W19_SCORE","#"],
                  ["SED_SN_SCORE","#"],
                  ["RCK_W_SCORE","#"],
                  ["RCK_SN_SCORE","#"],
                  ["HMC_W_SCORE","#"],
                  ["HMC_SN_SCORE","#"],
                  ["MIN_SCH_SCORE","#"],
                  ["MIN_CAS_SCORE","#"],
                  ]

# List of point datasets used
  datasets = [["ardf",ardf_points],["sed",sed_points],["rck",rck_points],["hmc",hmc_points],["min",mineral_points]]

# List of point datasets and elements for setting not analyzed values
  datasets2 = [["sed","sed_pnts_lyr","W_ppm > -9999 or Sn_ppm > -9999"],
               ["rck","rck_pnts_lyr","W_ppm > -9999 or Sn_ppm > -9999"],
               ["hmc","hmc_pnts_lyr","W_ppm > -9999 or Sn_ppm > -9999"]
               ]
   
# Lists of queries and datasets to loop through and assign scores to HUCs
# For scores being assigned to the same field the lowest score must always be listed
# first. This ensures that higher scores will always replace lower ones.

  mrat_elements = [    
     # SED queries
     ['sed_pnts_lyr',"(W_ppm >= 20 and W_ppm < 50)","SED_W_SCORE","4"],
     ['sed_pnts_lyr',"(W_ppm >= 50 and W_ppm < 100)","SED_W_SCORE","6"],    
     ['sed_pnts_lyr',"W_ppm >= 100","SED_W_SCORE","10"],

     ['sed_pnts_lyr1',"(W_ppm_1 >= 5 and W_ppm_1 < 10)","SED_W19_SCORE","4"],
     ['sed_pnts_lyr1',"(W_ppm_1 >= 10 and W_ppm_1 < 20)","SED_W19_SCORE","6"],
     ['sed_pnts_lyr1',"W_ppm_1 >= 20","SED_W19_SCORE","10"],

     ['sed_pnts_lyr',"(Sn_ppm >= 20 and Sn_ppm < 50)","SED_SN_SCORE","2"],     
     ['sed_pnts_lyr',"(Sn_ppm >= 50 and Sn_ppm < 100)","SED_SN_SCORE","3"],     
     ['sed_pnts_lyr',"Sn_ppm >= 100","SED_SN_SCORE","5"],


    # ROCK queries
     ['rck_pnts_lyr',"(W_ppm >= 20 and W_ppm < 50)","RCK_W_SCORE","2"],
     ['rck_pnts_lyr',"(W_ppm >= 50 and W_ppm < 100)","RCK_W_SCORE","5"],
     ['rck_pnts_lyr',"W_ppm >= 100","RCK_W_SCORE","8"],

     ['rck_pnts_lyr',"(Sn_ppm >= 20 and Sn_ppm < 50)","RCK_SN_SCORE","1"],
     ['rck_pnts_lyr',"(Sn_ppm >= 50 and Sn_ppm < 100)","RCK_SN_SCORE","2"],
     ['rck_pnts_lyr',"Sn_ppm >= 100","RCK_SN_SCORE","4"],                       

     # Heavy mineral concentrates
     ['hmc_pnts_lyr',"(W_ppm >= 50 and W_ppm < 100)","HMC_W_SCORE","4"],
     ['hmc_pnts_lyr',"(W_ppm >= 100 and W_ppm < 1000)","HMC_W_SCORE","6"],
     ['hmc_pnts_lyr',"W_ppm >= 1000","HMC_W_SCORE","10"],

     ['hmc_pnts_lyr',"(Sn_ppm >= 50 and Sn_ppm < 100)","HMC_SN_SCORE","2"],
     ['hmc_pnts_lyr',"(Sn_ppm >= 100 and Sn_ppm < 1000)","HMC_SN_SCORE","3"],
     ['hmc_pnts_lyr',"Sn_ppm >= 1000","HMC_SN_SCORE","5"],  
     
     # Heavy mineral concentrate mineralogy
     ['min_pnts_lyr',"SCH = 'moderate' OR SCH = 'present' AND\
                      (OreRelatedMnrl_Comment LIKE '_ g SCH%' OR\
                       OreRelatedMnrl_Comment LIKE '%SCH _ g%' OR\
                       OreRelatedMnrl_Comment LIKE '__ g SCH%' OR\
                       OreRelatedMnrl_Comment LIKE '%SCH __ g%' OR\
                       OreRelatedMnrl_Comment LIKE '__+ g SCH%') AND NOT\
                      (OreRelatedMnrl_Comment LIKE '%50+ g SCH%' OR OreRelatedMnrl_Comment LIKE '%SCH 80 g%' OR OreRelatedMnrl_Comment LIKE '%SCH 65 g%')","MIN_SCH_SCORE","3"],
     
     ['min_pnts_lyr',"SCH = 'abundant' OR SCH LIKE '%>%' OR SCH = '10-25%' OR SCH = '5-20%' OR\
                       OreRelatedMnrl_Comment LIKE '%10_ SCH%' OR\
                       OreRelatedMnrl_Comment LIKE '100+ g SCH%' OR\
                       OreRelatedMnrl_Comment LIKE '50+ g SCH%' OR\
                       OreRelatedMnrl_Comment LIKE '%SCH _00 g%'","MIN_SCH_SCORE","10"],
  
     ['min_pnts_lyr',"CAS = 'present' OR CAS = '1-5%'","MIN_CAS_SCORE","2"],
     
     ['min_pnts_lyr',"CAS = 'abundant' OR CAS = '5-20%'","MIN_CAS_SCORE","5"]]
##############################################################################

# Copy point feature classes into memory
  print "Copying datasets into memory..."
##  arcpy.CopyFeatures_management (ardf_points, "in_memory/ardf_pnts")
##  arcpy.CopyFeatures_management (sed_points, "in_memory/sed_pnts")
##  arcpy.CopyFeatures_management (rck_points, "in_memory/rck_pnts")
##  arcpy.CopyFeatures_management (hmc_points, "in_memory/hmc_pnts")
##  arcpy.CopyFeatures_management (mineral_points, "in_memory/mineral_pnts")

# Create temporary feature layers for input datasets so selections can be performed
  print "Creating feature layers..."
##  arcpy.MakeFeatureLayer_management(temp_path+"ardf_pnts","ardf_pnts_lyr")
##  arcpy.MakeFeatureLayer_management(temp_path+"sed_pnts","sed_pnts_lyr")
##  arcpy.MakeFeatureLayer_management(temp_path+"rck_pnts","rck_pnts_lyr")
##  arcpy.MakeFeatureLayer_management(temp_path+"hmc_pnts","hmc_pnts_lyr")
##  arcpy.MakeFeatureLayer_management(temp_path+"mineral_pnts","min_pnts_lyr")
  arcpy.MakeFeatureLayer_management (ardf_points, "ardf_pnts_lyr")
  arcpy.MakeFeatureLayer_management (sed_points, "sed_pnts_lyr")
  arcpy.MakeFeatureLayer_management (sed_points1, "sed_pnts_lyr1")
  arcpy.MakeFeatureLayer_management (rck_points, "rck_pnts_lyr")
  arcpy.MakeFeatureLayer_management (hmc_points, "hmc_pnts_lyr")
  arcpy.MakeFeatureLayer_management (mineral_points, "min_pnts_lyr")
 
##############################################################################

# Copy template tract polygons into memory
  arcpy.Delete_management("tract_polys_lyr")
  arcpy.Delete_management(tract_polys)
  arcpy.CopyFeatures_management (tract_polys_orig, tract_polys)
  arcpy.MakeFeatureLayer_management(tract_polys,"tract_polys_lyr","HUC_ID <> '0'")
  
# Add new fields to tract polygons
  for field in score_fields:
    arcpy.AddField_management(tract_polys,field[0], "SHORT","#","#","#",field[1])
    arcpy.CalculateField_management(tract_polys,field[0],"0","PYTHON")

  arcpy.AddField_management(tract_polys,"MAX_GEOCHEM", "TEXT","#","#","20")
  arcpy.AddField_management(tract_polys,"POTENTIAL", "TEXT","#","#","10")
  arcpy.AddField_management(tract_polys,"CERTAINTY", "TEXT","#","#","10")
  arcpy.AddField_management(tract_polys,"SYMBOL", "TEXT","#","#","20")
##############################################################################

# Join max ranking from ARDF dataset to polygons
  print "Setting ARDF values..."
  arcpy.SpatialJoin_analysis("tract_polys_lyr","ardf_pnts_lyr","in_memory/tract_ardf","JOIN_ONE_TO_ONE","KEEP_ALL",
                             'HUC_ID "HUC_ID" true true false 15 Text 0 0 ,First,#,tract_polys_lyr,HUC_ID,-1,-1;\
                             RT_RANKING "RT RANKING" true true false 4 Long 0 0 ,Max,#,ardf_pnts_lyr,RT_RANKING,-1,-1',"INTERSECT","","")
  
  arcpy.JoinField_management("tract_polys_lyr", "HUC_ID", "in_memory/tract_ardf", "HUC_ID", ["RT_RANKING"])

  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","NEW_SELECTION","RT_RANKING > 0") 
  arcpy.CalculateField_management("tract_polys_lyr","ARDF_SCORE","!RT_RANKING!","PYTHON")
  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")
##############################################################################
  
# Loop through list of point datasets to find tracts containing selected points and assign correct scores
# Scores for each query are added together except for temp fields where higher scores replace lower scores
  for elem in mrat_elements:
    print "Selecting "+elem[0]+" points..."
    # Find points that should be assigned a score
    arcpy.SelectLayerByAttribute_management(elem[0],"NEW_SELECTION",elem[1])
    count = int(arcpy.GetCount_management(elem[0]).getOutput(0))
    # Calculate score
    if (count > 0):
      arcpy.SelectLayerByLocation_management("tract_polys_lyr", "INTERSECT", elem[0],"#","NEW_SELECTION")
      # Replace lower scores in field with higher ones
      arcpy.CalculateField_management("tract_polys_lyr",elem[2],elem[3],"PYTHON")
    arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")
    arcpy.SelectLayerByAttribute_management(elem[0],"CLEAR_SELECTION")
##############################################################################

# After scoring is completed calculate values for remaining fields
  print "Calculating totals and certainty..."
  arcpy.CalculateField_management("tract_polys_lyr","SED_SCORE","!SED_W_SCORE!+!SED_SN_SCORE!","PYTHON")
  arcpy.CalculateField_management("tract_polys_lyr","SED_19_SCORE","!SED_W19_SCORE!","PYTHON")
  arcpy.CalculateField_management("tract_polys_lyr","RCK_SCORE","!RCK_W_SCORE!+!RCK_SN_SCORE!","PYTHON")
  arcpy.CalculateField_management("tract_polys_lyr","HMC_SCORE","!HMC_W_SCORE!+!HMC_SN_SCORE!","PYTHON")
  arcpy.CalculateField_management("tract_polys_lyr","MIN_SCORE","!MIN_SCH_SCORE!+!MIN_CAS_SCORE!","PYTHON")
  arcpy.CalculateField_management("tract_polys_lyr","TOTAL_SCORE","!ARDF_SCORE! + max(!SED_SCORE!,!SED_W19_SCORE!,!RCK_SCORE!,!HMC_SCORE!,!MIN_SCORE!)","PYTHON")

# Identify the geochem dataset(s) with the highest score 
  with arcpy.da.UpdateCursor("tract_polys_lyr", ["ARDF_SCORE","SED_SCORE","SED_19_SCORE","RCK_SCORE","HMC_SCORE","MIN_SCORE","TOTAL_SCORE","MAX_GEOCHEM"]) as cursor:
    for row in cursor:
      max_value = row[6] - row[0]
      row[7] = ''
      if max_value > 0:
        if (row[1] == max_value):
          row[7] = row[7] + 'SED'
        if (row[2] == max_value):
          row[7] = row[7] + 'SED19'  
        if (row[3] == max_value):
          row[7] = row[7] + '; RCK'
        if (row[4] == max_value):
          row[7] = row[7] + '; HMC'
        if (row[5] == max_value):
          row[7] = row[7] + '; MIN'
        row[7] = row[7].lstrip(";")
        row[7] = row[7].lstrip()
      cursor.updateRow(row)

# Calculate certainty for ARDF and max geochem dataset
  with arcpy.da.UpdateCursor("tract_polys_lyr", ["ARDF_SCORE","SED_W_SCORE","SED_SN_SCORE","RCK_W_SCORE","RCK_SN_SCORE","HMC_W_SCORE","HMC_SN_SCORE","MIN_SCH_SCORE","MIN_CAS_SCORE","CERTAINTY","MAX_GEOCHEM","TOTAL_SCORE","POTENTIAL","SED_19_SCORE"]) as cursor:
    for row in cursor:
      value = 0
      # Certainty values for the geochemistry
      max_dataset = row[10].split(";")
      if (row[2] > 0 and max_dataset[0] == "SED"):
        value = 3
      if (row[1] > 0 and max_dataset[0] == "SED"):
        value = value + 4

      if (row[13] > 0 and max_dataset[0] == "SED19"):
        value = 4  
      
      if (row[4] > 0 and max_dataset[0] == "RCK"):
        value = 2  
      if (row[3] > 0 and max_dataset[0] == "RCK"):
        value = value + 3
      
      if (row[6] > 0 and max_dataset[0] == "HMC"):
        value = 3 
      if (row[5] > 0 and max_dataset[0] == "HMC"):
        value = value + 4
      
      if (row[8] > 0 and max_dataset[0] == "MIN"):
        value = 2  
      if (row[7] > 0 and max_dataset[0] == "MIN"):
        value = value + 3
      # Certainty value for ARDF
      if (row[0] > 0):
        value = value + 5
      # Classify the certainty value
      if (value < 4):
        cert = "LOW"
      if (value >= 4 and value < 9):
        cert = "MED"
      if (value >= 9):
        cert = "HIGH"
      row[9] = cert

      if (row[11] >= 0 and row[11] <= 3):
        potential = "LOW"
      if (row[11] >= 4 and row[11] <= 10):
        potential = "MED"
      if (row[11] >= 11):
        potential = "HIGH"
      row[12] = potential
      cursor.updateRow(row)
      
# Loop through list of point datasets to find HUCs containing no data and set them to null
  for data in datasets:
    arcpy.SelectLayerByLocation_management("tract_polys_lyr","INTERSECT",data[1],"","NEW_SELECTION")
    arcpy.SelectLayerByLocation_management("tract_polys_lyr","","","","SWITCH_SELECTION")
    arcpy.CalculateField_management("tract_polys_lyr",data[0]+"_SCORE","None","PYTHON")
    arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")

# Loop through list of point datasets to find HUCs with no score for that dataset and if
# all the points in a HUC were not analyzed for selected elements set value to -9999
  print "Setting not analyzed values..."
  for data in datasets2:
    arcpy.SelectLayerByAttribute_management(data[1],"CLEAR_SELECTION")
    arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")
    
    arcpy.SelectLayerByAttribute_management(data[1],"NEW_SELECTION",data[2])
    arcpy.SelectLayerByAttribute_management("tract_polys_lyr","NEW_SELECTION",data[0]+"_SCORE = 0")
    arcpy.SelectLayerByLocation_management("tract_polys_lyr","INTERSECT",data[1],"","REMOVE_FROM_SELECTION")    
    arcpy.CalculateField_management("tract_polys_lyr",data[0]+"_SCORE","-9999","PYTHON")

# Calculate the symbol field
  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")
##  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","NEW_SELECTION","SED_SCORE = 0 AND RCK_SCORE = 0 AND HMC_SCORE = 0 AND MIN_SCORE = 0 AND TOTAL_SCORE = 0")
##  arcpy.CalculateField_management("tract_polys_lyr","CERTAINTY","\"HIGH\"","PYTHON_9.3")
##  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")
  arcpy.CalculateField_management("tract_polys_lyr","SYMBOL","""!POTENTIAL! +","+ !CERTAINTY!""","PYTHON_9.3")
  
# Identify HUCs that should be UNKNOWN
  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")  
  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","NEW_SELECTION","(SED_SCORE IS NULL and TOTAL_SCORE = 0)")
  arcpy.CalculateField_management("tract_polys_lyr","POTENTIAL","\"UNKNOWN\"","PYTHON_9.3")
  arcpy.CalculateField_management("tract_polys_lyr","CERTAINTY","\"UNKNOWN\"","PYTHON_9.3")
  arcpy.CalculateField_management("tract_polys_lyr","SYMBOL","\"UNKNOWN\"","PYTHON_9.3")
  arcpy.SelectLayerByAttribute_management("tract_polys_lyr","CLEAR_SELECTION")

# Delete temporary scoring fields
  arcpy.DeleteField_management(tract_polys,["FID_W_skarn_tractfinal","FID_HUC12_state","RT_RANKING",
                                            "HUC12","ToHUC",
##                                          "SED_W_SCORE","SED_SN_SCORE","RCK_W_SCORE","RCK_SN_SCORE","HMC_W_SCORE","HMC_SN_SCORE","MIN_SCH_SCORE","MIN_CAS_SCORE",
                                          ])

# Copy HUCs from memory to disk
  arcpy.CopyFeatures_management (tract_polys, output_path+"tmp5")

##############################################################################
  
# In case of error exit and give error message.
except arcpy.ExecuteError: 
    msgs = arcpy.GetMessages(2) 
    arcpy.AddError(msgs) 
    print msgs

except:
    tb = sys.exc_info()[2]
    tbinfo = traceback.format_tb(tb)[0]
    pymsg = "PYTHON ERRORS:\nTraceback info:\n" + tbinfo + "\nError Info:\n" + str(sys.exc_info()[1])
    msgs = "ArcPy ERRORS:\n" + arcpy.GetMessages(2) + "\n"
    arcpy.AddError(pymsg)
    arcpy.AddError(msgs)
    print pymsg + "\n"
    print msgs
